/*
	Tabulatorweite: 3
	Kommentare ab: Spalte 60											*Spalte 60*
*/

#include	<mgx_dos.h>
#include	<stddef.h>
#include	<string.h>
#include <portab.h>
#include <toserror.h>
#include "imgload.h"

typedef struct
{
	WORD	version;
	WORD	length;
	WORD	planes;
	WORD	pattern_length;
	WORD	pix_width;
	WORD	pix_height;
	WORD	w;
	WORD	h;
} IMGHDR;

static LONG open_img( char *name );
static WORD close_img( void );
static void unpack_line( UBYTE *des );
static void fill_img_buf( void );
static UBYTE *unpack_line1( UBYTE *img, UBYTE *des, WORD pat_len, WORD len );
static UBYTE *unpack_line2( UBYTE *img, UBYTE *des, WORD pat_len, WORD len );

static UBYTE *img_buf;			/* Zeiger auf IMG-Buffer */
static WORD unpacked_lines;		/* Anzahl der entpackten Zeilen */
static LONG buf_len;			/* Bufferlnge */
static LONG buf_used;			/* Anzahl der benutzten Bytes */
static LONG buf_offset;			/* Abstand zum Bufferanfang */
static WORD fd;				/* GEMDOS-Handle des IMGs */
static void *external_buf;
static WORD pat_len;			/* Lnge eines Musters in Bytes */
static WORD line_len;			/* Lnge einer Zeile in Bytes */
static WORD img_pl;
static LONG d_len;				/* verbleibende Dateilnge */

static IMGHDR	head;

/*--------------------------------------------------------------------------------*/ 
/* IMG laden und entpacken																						*/
/* Funktionsresultat:	Zeiger auf das entpackte IMG (liegt im Standardformat vor!)			*/
/*	name:						Dateiname																			*/
/*	w:							hier wird die Breite in Pixeln zurckgeliefert							*/
/*	h:							hier wird die Hhe in Zeilen zurckgeliefert								*/
/*	line_width:				hier wird die Zeilenbreite in Bytes zurckgeliefert					*/
/*	planes:					hier wird die Anzahl der Ebenen zurckgeliefert							*/
/* palette:					Zeiger auf Zeiger auf Palettendaten (0: keine Palette vorhanden)	*/
/*	pal_entries:			Anzahl der Eintrge in der Palette 											*/
/*----------------------------------------------------------------------------------------*/ 
LONG load_IMG( BYTE *name, WORD *w, WORD *h,
			WORD *line_width, WORD *planes,
			WORD **palette, WORD *pal_entries,
			UBYTE **img )
{
	LONG retcode;
	UBYTE	*buf;
	UBYTE	*des;
	LONG	plane_len;


	*img = NULL;
	retcode = open_img( (char *)name );
	if	(retcode < 0)
		return(retcode);

	*w = head.w;														/* Breite in Pixeln */
	*h = head.h;														/* Hhe in Zeilen */
	*line_width = (( head.w + 15 ) & 0xfff0) / 8;	/* Breite in Bytes (Vielfaches von 2) */
	*planes = head.planes;											/* Anzahl der Ebenen */
	*palette = 0L;														/* keine Palettendaten */
	*pal_entries = 0;													/* keine Eintrge */

	plane_len = (LONG) ( *line_width ) * head.h;		/* Lnge einer Ebene */

	if	( head.length > ( sizeof(IMGHDR) / 2 ))		/* mglicherweise XIMG? */
		{
		BYTE	identify[6];

		Fseek( sizeof( IMGHDR ), fd, 0 );					/* Dateiheader berspringen */
		Fread( fd, 6, identify );
		if	( *(LONG *) identify == 'XIMG' )						/* XIMG-Kennung vorhanden? */
			{
			LONG	pal_len;
			
			pal_len = (head.length * 2 ) - 6 - sizeof( IMGHDR );	/* Lnge der Palette in Bytes */
			
			if ( pal_len > ( 256 * 6 ))
				pal_len = 256 * 6;
			
			*palette = Malloc( pal_len );							/* Speicher fr Palette anfordern */
			if ( *palette )
			{
				*pal_entries = (WORD) pal_len / 6;				/* Anzahl der Paletteneintrge */
				Fread( fd, pal_len, *palette );				/* Palettendaten einlesen */
			}
			}
		Fseek( head.length * 2, fd, 0 );					/* Dateiheader berspringen */
		}

	des = Malloc( plane_len * head.planes );					/* Buffer fr entpacktes Bild anfordern */
	buf = Malloc((LONG) (*line_width) * head.planes );		/* Buffer frs Entpacken einer Zeile */

	if	(( buf == 0L ) && ( des ))									/* konnte der Zeilenbuffer nicht angefordert werden? */
		{
		Mfree( des );
		des = 0L;
		}	
	
	if	( des )
		{
		WORD	y;
		UBYTE	*tmp;
		
		tmp = des;
		
		for	( y = 0; y < head.h; y++ )
			{
			WORD	plane;
			UBYTE	*src;
			UBYTE	*dst;
			
			unpack_line( buf );										/* komplette Zeile in den Buffer entpacken */
			
			src = buf;
			dst = tmp;				
			
			for ( plane = 0; plane < head.planes; plane++ )
			{
				memcpy( dst, src, line_len );						/* Ebene in den Bildbuffer kopieren */
				src += line_len;
				dst += plane_len;
			}
			
			tmp += *line_width;										/* nchste Zeile */
			}
		}

	if ( buf )
		Mfree( buf );													/* Zeilenbuffer freigeben */

	close_img();

	*img = des;
	return( E_OK );
}

/*----------------------------------------------------------------------------------------*/ 
/* IMG-Datei ffnen																								*/
/* Funktionsresultat:	1: alles in Ordung 0: Fehler													*/
/* name:						Dateiname																			*/
/*----------------------------------------------------------------------------------------*/ 
static LONG open_img( char *path )
{
	XATTR xa;
	LONG retcode;


	retcode = Fopen( path, FO_READ );
	if	(retcode < 0)
		return(retcode);
	fd = (WORD) retcode;
	retcode = Fcntl(fd, (long) &xa, FSTAT);
	if	(!retcode)
		d_len = xa.st_size;
	else	goto err;

	external_buf = Malloc( d_len );	/* Puffer anfordern */

	if	( !external_buf )
		{
		retcode = ENSMEM;
		err:
		Fclose(fd);
		return(retcode);
		}

	img_buf = external_buf;
	buf_len = d_len;
			
	Fread( fd, sizeof( IMGHDR ), &head );
	Fseek( head.length * 2, fd, 0 );		/* Dateiheader berspringen */
	d_len -= head.length * 2;		/* verbleibende Dateilnge korrigieren */
	fill_img_buf();				/* Buffer auffllen */
	pat_len = head.pattern_length;	/* Bytes pro Muster */
	line_len = ( head.w + 7 ) / 8;	/* Lnge einer IMG-Zeile in Bytes */
	img_pl = head.planes;
	unpacked_lines = 0;				/* noch keine Zeile entpackt */

	return( E_OK );
}

/*----------------------------------------------------------------------------------------*/ 
/* IMG-Datei schlieen																							*/
/* Funktionsresultat:	1																						*/
/*----------------------------------------------------------------------------------------*/ 
static WORD close_img( void )
{
	if ( external_buf )													/* Buffer ber Malloc alloziert? */
		Mfree( external_buf );

	Fclose( fd );
	return( 1 );
}

/*----------------------------------------------------------------------------------------*/ 
/* IMG-Zeile auspacken																							*/
/* Funktionsresultat:	-																						*/
/*----------------------------------------------------------------------------------------*/ 
static void unpack_line( UBYTE *des )
{
	UBYTE	*img_line;
	LONG	read;
	
	if ( unpacked_lines < head.h )									/* sind noch gepackte Zeilen vorhanden? */
	{
		img_line = img_buf + buf_offset;
		read = (LONG) ( unpack_line1( img_line, des, pat_len, line_len * head.planes ) - img_line );
		buf_used -= read;													/* Anzahl der gltigen Bytes im Buffer */
		buf_offset += read;												/* Abstand zum Bufferstart */
		unpacked_lines++;													/* eine weitere Zeile ist entpackt worden */
	}
}

/*----------------------------------------------------------------------------------------*/ 
/* Buffer zum ersten Mal fllen																				*/
/* Funktionsresultat:	-																						*/
/*----------------------------------------------------------------------------------------*/ 
static void fill_img_buf( void )
{
	LONG	read;

	read = buf_len;														/* Buffer komplett fllen */
	if ( read > d_len )													/* mehr Bytes als die Datei lang ist? */
		read = d_len;	

	buf_offset = 0;														/* Bufferstart */	
	buf_used = Fread( fd, read, img_buf );	/* einlesen */
	d_len -= buf_used;													/* noch vorhandene Dateilnge verkleinern */
}

/*----------------------------------------------------------------------------------------*/ 
/* Zeile eines IMGs entpacken																					*/
/* Funktionsresultat:	Zeiger auf die nchste Zeile im IMG-Format								*/
/*	img:						Zeiger auf die IMG-Zeile														*/
/*	des:						Zeiger auf die Zielbitmap														*/
/*	pat_len:					Lnge eines IMG-Muster in Bytes												*/
/*	len:						Lnge einer entpackten Zeile in Bytes										*/
/*----------------------------------------------------------------------------------------*/ 
static UBYTE *unpack_line1( UBYTE *img, UBYTE *des, WORD pat_len, WORD len )
{
	if (( img[0] == 0 ) && ( img[1] == 0 ) && ( img[2] == 0xff ))	/* vertikale Wiederholung? */
	{
		if ( img[3] > 1 )													/* mehr als eine Zeile? */
		{
			unpack_line2( img + 4, des, pat_len, len );
			img[3] -= 1;													/* eine Wiederholung weniger */
			return( img );
		}
		else
			return( unpack_line2( img + 4, des, pat_len, len ));
	}
	else
		return( unpack_line2( img, des, pat_len, len ));
}

/*----------------------------------------------------------------------------------------*/ 
/* Zeile eines IMGs entpacken, vertikale Wiederholfaktoren drfen nicht vorkommen			*/
/* Funktionsresultat:	Zeiger auf die nchste Zeile im IMG-Format								*/
/*	img:						Zeiger auf die IMG-Zeile														*/
/*	des:						Zeiger auf die Zielbitmap														*/
/*	pat_len:					Lnge eines IMG-Muster in Bytes												*/
/*	len:						Lnge einer entpackten Zeile in Bytes										*/
/*----------------------------------------------------------------------------------------*/ 
static UBYTE *unpack_line2( UBYTE *img, UBYTE *des, WORD pat_len, WORD len )
{
	WORD	i;
	UWORD	cnt;

	while ( len > 0 )														/* komplette Zeile abgearbeitet? */
	{
		UBYTE	tag;
		
		tag = *img++;
		
		if ( tag == 0 )													/* Pattern Run? */
		{
			cnt = *img++;													/* Anzahl der Wiederholungen */
			
			for ( i = 0; i < cnt; i++ )
			{
				WORD	j;
				for ( j = 0; j < pat_len; j++ )
					*des++ = img[j];
			}
																							
			img += pat_len;												/* Musterdaten berspringen */
			cnt *= pat_len;												/* Lnge des Musters */
		}
		else if ( tag == 0x80 )											/* Bit String? */
		{
			cnt = *img++;													/* Anzahl der unkomprimierten Bytes */
		
			for ( i = 0; i < cnt; i++ )
				*des++ = *img++;
		}
		else if (( tag & 0x80 ) == 0 )								/* weier Solid Run? */
		{
			cnt = tag & 0x7f;												/* Anzahl der Wiederholungen */
			
			for ( i = 0; i < cnt; i++ )
				*des++ = 0;
		}
		else																	/* schwarzer Solid Run */
		{
			cnt = tag & 0x7f;												/* Anzahl der Wiederholungen */
			
			for ( i = 0; i < cnt; i++ )
				*des++ = 0xff;
		}	

		len -= cnt;															/* Anzahl der noch vorhandenen Bytes */
	}
	return( img );
}
